// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package application

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkApplicationOrderAmountParam(req *pb_v1.ApplicationOrderAmountRequest) error {
	switch {
	case req.OrderId == 0:
		return status.Error(10003, "订单不能为空错误")
	}
	return nil
}

//
func ApplicationOrderAmount(ctx context.Context, req *pb_v1.ApplicationOrderAmountRequest) (reply *pb_v1.ApplicationOrderAmountReply, err error) {

	reply = &pb_v1.ApplicationOrderAmountReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkApplicationOrderAmountParam(req)
	if err != nil {
		return nil, err
	}

	now := time.Now()

	p := dbmodel.TApplicationOrder{}
	where := map[string]interface{}{
		"id": req.OrderId,
	}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	if p.Status != 1 {
		return nil, status.Error(10003, "当前状态不能改价")
	}

	values := map[string]interface{}{
		"amount":     req.Amount,
		"updated_at": now,
	}
	err = p.Update(database.DB(), where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}

	return reply, nil
}
