// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package department

import (
	"context"
	"encoding/json"
	"fmt"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"strings"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkDepartmentAddParam(req *pb_v1.DepartmentAddRequest) error {
	switch {
	case req.DepartmentName == "":
		return status.Error(10003, "部门名不能为空")
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.Cid < 1:
		return status.Error(10003, "物业公司不能为空")

	}
	return nil
}


//
func DepartmentAdd(ctx context.Context, req *pb_v1.DepartmentAddRequest) (reply *pb_v1.DepartmentAddReply, err error) {

	reply = &pb_v1.DepartmentAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkDepartmentAddParam(req)
	if err != nil {
		return nil, err
	}

	now := time.Now()
	p := dbmodel.TDepartment{
		GardenId:req.GardenId,
		Cid:            req.Cid,
		DepartmentName:req.DepartmentName,
		Desc:req.Desc,
		UpdatedAt:      now,
		CreatedAt:      now,
	}

	err = p.Insert(database.DB())
	if err != nil {
		if strings.Contains(strings.ToLower(err.Error()), "duplicate") {
			return nil, status.Error(10003, "该部门已存在")
		}
		return nil, errors.DataBaseError
	}
	reply.Id = p.ID

	return reply, nil
}

