// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package department

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)


//
func DepartmentDel(ctx context.Context, req *pb_v1.DepartmentDelRequest) (reply *pb_v1.DepartmentDelReply, err error) {

	reply = &pb_v1.DepartmentDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.Id < 1 {
		return nil, errors.ParamsError
	}

	if req.GardenId < 1 {
		return nil, errors.ParamsError
	}

	old := &dbmodel.TDepartment{}
	where := map[string]interface{}{
		"id":req.Id,
		"garden_id":req.GardenId,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	reply.Origin = &pb_v1.DepartmentUpdateRequest{
	Id:req.Id,
	GardenId:old.GardenId,
	Desc:old.Desc,
	DepartmentName:old.DepartmentName,
	}

	db := database.DB().Begin()
	defer func() {
		if err != nil {
			db.Rollback()
		}
	}()

	err = old.Delete(db, where)
	if err != nil {
		return nil, errors.DataBaseError
	}
	user := &dbmodel.TUser{}
	where = map[string]interface{}{
		"department_id":req.Id,
	}
	values := map[string]interface{}{
		"department_id":0,
	}
	err = user.Update(db, where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}
	db.Commit()

	return reply, nil
}

