// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package department

import (
	"context"
	"encoding/json"
	"fmt"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

//
func DepartmentList(ctx context.Context, req *pb_v1.DepartmentListRequest) (reply *pb_v1.DepartmentListReply, err error) {

	reply = &pb_v1.DepartmentListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.GardenId == 0 {
		return nil, errors.ParamsError
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}
	reply.Page = req.Page


	p := &dbmodel.TDepartment{}
	where := map[string]interface{}{
		"garden_id":req.GardenId,
	}

	reply.Total, err = p.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := p.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.List = make([]*pb_v1.DepartmentItem, len(list))

	for i, v := range list {
		item:= &pb_v1.DepartmentItem{
			Id:v.ID,
			DepartmentName:v.DepartmentName,
			Desc:v.Desc,
			CreatedAt:v.CreatedAt.Format("2006-01-02 15:04:05"),
		}
		reply.List[i] = item
	}


	return reply, nil
}

