// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package department

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"time"
)

func checkDepartmentUpdateParam(req *pb_v1.DepartmentUpdateRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.Id < 1:
		return status.Error(10003, "用户id不能为空")
	case req.DepartmentName == "" :
		return status.Error(10003, "部门名称不能为空")

	}
	return nil
}

//
func DepartmentUpdate(ctx context.Context, req *pb_v1.DepartmentUpdateRequest) (reply *pb_v1.DepartmentUpdateReply, err error) {

	reply = &pb_v1.DepartmentUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkDepartmentUpdateParam(req)
	if err != nil {
		return nil, err
	}
	old := &dbmodel.TDepartment{}
	where := map[string]interface{}{
		"id":req.Id,
		"garden_id":req.GardenId,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	reply.Origin = &pb_v1.DepartmentUpdateRequest{
		Id:req.Id,
		GardenId:old.GardenId,
		Desc:old.Desc,
		DepartmentName:old.DepartmentName,
	}
	now := time.Now()

	values := map[string]interface{}{
		"department_name":req.DepartmentName,
		"desc":req.Desc,
		"updated_at":now,
	}
	err = old.Update(database.DB(), where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}


	return reply, nil
}
