// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package garden

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-system/errors"
	"property-system/impl/v1/permission"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkGardenEnableSetParam(req *pb_v1.GardenEnableSetRequest) error {
	switch {
	case req.GardenId == 0:
		return status.Error(10003, "小区id不能为空")
	}
	return nil

}

//
func GardenEnableSet(ctx context.Context, req *pb_v1.GardenEnableSetRequest) (reply *pb_v1.GardenEnableSetReply, err error) {

	reply = &pb_v1.GardenEnableSetReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkGardenEnableSetParam(req)
	if err != nil {
		return nil, err
	}
	now := time.Now()
	values := map[string]interface{}{
		"updated_at": now,
		"enable":     1,
	}
	if !req.Enable {
		values["enable"] = 2
	}
	where := map[string]interface{}{
		"id": req.GardenId,
	}
	p := dbmodel.TGardenApproved{}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	err = p.Update(database.DB(), where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}
	permission.GlobalPermissionTimeCache(req.GardenId)

	return reply, nil
}
