// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package garden

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-system/errors"

	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

//
func GardenHouseCountChange(ctx context.Context, req *pb_v1.GardenHouseCountChangeRequest) (reply *pb_v1.GardenHouseCountChangeReply, err error) {

	reply = &pb_v1.GardenHouseCountChangeReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.GardenId == 0 || req.Count == 0 {
		return nil, errors.ParamsError
	}

	garden := &dbmodel.TGardenApproved{}
	where := map[string]interface{}{
		"in_use": true,
		"id":     req.GardenId,
	}
	err = garden.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if garden.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	if garden.HouseCount+req.Count > garden.HouseCountLimit {
		return nil, status.Error(10003, "房屋数超过套餐限制")
	}
	values := map[string]interface{}{
		"house_count": gorm.Expr("house_count + ?", req.Count),
	}
	if req.Count < 0 {
		values["house_count"] = gorm.Expr("house_count - ?", -req.Count)
	}
	err = garden.Update(database.DB(), where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}

	return reply, nil
}
