// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package garden

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-system/errors"

	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

//
func GardenHouseCountLimit(ctx context.Context, req *pb_v1.GardenHouseCountLimitRequest) (reply *pb_v1.GardenHouseCountLimitReply, err error) {

	reply = &pb_v1.GardenHouseCountLimitReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	if req.GardenId == 0 {
		return nil, errors.ParamsError
	}
	garden := &dbmodel.TGardenApproved{}
	where := map[string]interface{}{
		"in_use": true,
		"id":     req.GardenId,
	}
	err = garden.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if garden.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	reply.HouseCountLimit = garden.HouseCountLimit
	reply.HouseCount = garden.HouseCount

	return reply, nil
}
