// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package garden

import (
	"context"
	"encoding/json"
	"fmt"
	"time"

	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

//
func GardenIds(ctx context.Context, req *pb_v1.GardenIdsRequest) (reply *pb_v1.GardenIdsReply, err error) {

	reply = &pb_v1.GardenIdsReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	garden := &dbmodel.TGardenApproved{}
	where := map[string]interface{}{
		"in_use": true,
	}
	if req.GardenName != "" {
		where["garden_name like"] = "%" + req.GardenName + "%"
	}
	if req.BuildingType > 0 {
		where["building_type"] = req.BuildingType
	}

	if req.BuildingYearLess > 0 {
		year := time.Now().Year() - int(req.BuildingYearLess)
		where["building_end >="] = time.Date(year, 1, 1, 0, 0, 0, 0, time.Now().Location()).Unix()
	}
	if req.BuildingYearGreater > 0 {
		year := time.Now().Year() - int(req.BuildingYearGreater)
		where["building_start <="] = time.Date(year, 1, 1, 0, 0, 0, 0, time.Now().Location()).Unix()
	}
	type IdObj struct {
		Id         int64
		GardenName string `json:"garden_name"`
	}
	ids := []IdObj{}
	err = garden.SelectList(database.DB(), where, "id, garden_name", &ids)
	if err != nil {
		return nil, err
	}
	reply.GardenIds = make([]*pb_v1.GardenIdInfo, len(ids))
	for i, v := range ids {
		reply.GardenIds[i] = &pb_v1.GardenIdInfo{
			GardenId:   v.Id,
			GardenName: v.GardenName,
		}
	}

	return reply, nil
}
