// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package garden

import (
	"context"
	"encoding/json"
	"fmt"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

//
func CompanyMchGardenList(ctx context.Context, req *pb_v1.CompanyMchGardenListRequest) (reply *pb_v1.CompanyMchGardenListReply, err error) {

	reply = &pb_v1.CompanyMchGardenListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.Cid < 1 {
		return nil, errors.ParamsError
	}
	p := dbmodel.TGardenApproved{}
	where := map[string]interface{}{
		"cid": req.Cid,
	}
	if req.MchId != "" {
		where["mch_id"] = req.MchId
	}
	if req.GardenId > 0 {
		where["id"] = req.GardenId
	}

	list, err := p.ListColums(database.DB(), where, nil, []string{"id", "garden_name", "pay_mode", "mch_id"})
	if err != nil {
		return
	}
	reply.List = make([]*pb_v1.CompanyMchGardenItem, len(list))
	for i, v := range list {
		reply.List[i] = &pb_v1.CompanyMchGardenItem{
			GardenId:   v.ID,
			PayMode:    v.PayMode,
			GardenName: v.GardenName,
			MchId:      v.MchId,
		}
	}

	return reply, nil
}
