// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package msg_package

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
)

func checkMsgPackageOrderDecreaseParam(req *pb_v1.MsgPackageOrderDecreaseRequest) error {
	switch {
	case req.OrderId < 1:
		return status.Error(10003, "订单不能为空")
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.Count == 0:
		return status.Error(10003, "数量不能为空")

	}
	return nil
}

//
func MsgPackageOrderDecrease(ctx context.Context, req *pb_v1.MsgPackageOrderDecreaseRequest) (reply *pb_v1.MsgPackageOrderDecreaseReply, err error) {

	reply = &pb_v1.MsgPackageOrderDecreaseReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkMsgPackageOrderDecreaseParam(req)
	if err != nil {
		return nil, err
	}

	p := dbmodel.TMsgPackageOrder{}
	where := map[string]interface{}{
		"id":        req.OrderId,
		"garden_id": req.GardenId,
	}
	values := map[string]interface{}{
		"msg_available_count": gorm.Expr("msg_available_count - ?", req.Count),
	}
	err = p.Update(database.DB(), where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}

	return reply, nil
}
