// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package msg_package

import (
	"context"
	"encoding/json"
	"fmt"
	"property-system/errors"
	dbmodel "property-system/model"
	"property-system/parser"
	"property-system/pb"
	pb_v1 "property-system/pb/v1"
	"property-system/utils"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkMsgPackageOrderAddParam(req *pb_v1.MsgPackageOrderAddRequest) error {
	switch {
	case req.PackageId < 1:
		return status.Error(10003, "套餐不能为空")
	case req.GardenId < 1 || req.Cid < 1:
		return status.Error(10003, "小区和公司不能为空")
	case req.PackageCount == 0:
		return status.Error(10003, "套餐个数不能为空")

	}
	return nil
}

//
func MsgPackageOrderAdd(ctx context.Context, req *pb_v1.MsgPackageOrderAddRequest) (reply *pb_v1.MsgPackageOrderAddReply, err error) {

	reply = &pb_v1.MsgPackageOrderAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkMsgPackageOrderAddParam(req)
	if err != nil {
		return nil, err
	}

	now := time.Now()

	p := dbmodel.TMsgPackageOrder{}

	mreq := pb_v1.MsgPackageInfoRequest{Id: req.PackageId}
	mreply, err := pb.Common.MsgPackageInfo(ctx, &mreq)
	if err != nil {
		return nil, err
	}
	if mreply.Price*req.PackageCount != req.Amount {
		return nil, status.Error(10003, "金额不一致")
	}

	p.CreatedAt = now
	p.UpdatedAt = now
	p.PackageId = req.PackageId
	p.Amount = req.Amount
	p.GardenId = req.GardenId
	p.MsgCount = mreply.MsgCount
	p.Status = 1
	p.MsgTotalCount = mreply.MsgCount * req.PackageCount
	p.Name = mreply.Name
	p.Comment = mreply.Comment
	p.MsgAvailableCount = p.MsgTotalCount
	p.Price = mreply.Price
	p.Cid = req.Cid
	p.PackageCount = req.PackageCount

	db := database.DB().Begin()
	err = p.Insert(db)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	db.Commit()
	reply.OrderId = p.ID
	reply.PackageCount = req.PackageCount
	reply.Amount = p.Amount
	reply.GardenCount = 1
	reply.PayType = 1
	reply.BankName = parser.Conf.Bank.BankName
	reply.BankAccount = parser.Conf.Bank.BankAccount
	reply.BankUserName = parser.Conf.Bank.BankUserName
	utils.RobotMsg(fmt.Sprintf("有新的短信订单，请关注"))
	return reply, nil
}
