// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package permission

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"strings"
)

func checkGroupDelParam(req *pb_v1.GroupDelRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.Cid < 1:
		return status.Error(10003, "物业公司不能为空")
	case req.Id < 1:
		return status.Error(10003, "权限组不能为空")

	}
	return nil
}

//
func GroupDel(ctx context.Context, req *pb_v1.GroupDelRequest) (reply *pb_v1.GroupDelReply, err error) {

	reply = &pb_v1.GroupDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkGroupDelParam(req)
	if err != nil {
		return nil, err
	}
	p := dbmodel.TGroup{}
	where := map[string]interface{}{
		"id":req.Id,
		"cid":req.Cid,
		"garden_id":req.GardenId,
	}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound{
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	if p.IsSuper == 1 {
		return nil,  errors.SuperGroupError
	}
	reply.Origin = &pb_v1.GroupUpdateRequest{
		Id:req.Id,
		Cid:req.Cid,
		GardenId: req.GardenId,
		GroupName:p.GroupName,
		GroupDesc:p.GroupDesc,
		PermissionCodes:strings.Split(p.PermissionCodes, ","),
	}

	user := dbmodel.TUser{}
	where = map[string]interface{}{
		"group_id":req.Id,
		"cid":req.Cid,
		"garden_id":req.GardenId,
	}
	count, err := user.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if count > 0 {
		return nil, errors.GroupCanNotDel
	}

	group := &dbmodel.TGroup{}
	where = map[string]interface{}{
		"id":req.Id,
		"cid":req.Cid,
		"garden_id":req.GardenId,
	}
	err = group.Delete(database.DB(), where)

	if err != nil {
		return nil, errors.DataBaseError
	}

	return reply, nil
}

