// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package permission

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
)

func checkGroupListParam(req *pb_v1.GroupListRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.Cid < 1:
		return status.Error(10003, "物业公司不能为空")

	}
	return nil
}

//
func GroupList(ctx context.Context, req *pb_v1.GroupListRequest) (reply *pb_v1.GroupListReply, err error) {

	reply = &pb_v1.GroupListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkGroupListParam(req)
	if err != nil {
		return nil, err
	}
	p := dbmodel.TGroupUser{}
	where := map[string]interface{}{
		"cid":       req.Cid,
		"garden_id": req.GardenId,
	}
	list, err := p.List(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.List = make([]*pb_v1.GroupItem, len(list))
	for i, v := range list {
		item := &pb_v1.GroupItem{
			GroupName: v.GroupName,
			GroupDesc: v.GroupDesc,
			UserCount: v.UserCount,
			Id:        v.ID,
		}
		if v.IsSuper == 1 {
			item.Super = true
		}
		reply.List[i] = item
	}
	return reply, nil
}
