// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package permission

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)



//
func SuperGroup(ctx context.Context, req *pb_v1.SuperGroupRequest) (reply *pb_v1.SuperGroupReply, err error) {

	reply = &pb_v1.SuperGroupReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.GardenId == 0 || req.Cid == 0 {
		return nil, errors.ParamsError
	}

	p := dbmodel.TGroup{}
	where := map[string]interface{}{
		"cid":req.Cid,
		"garden_id":req.GardenId,
		"is_super":1,
	}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID > 0 {
		reply.Id = p.ID
		return reply, nil
	}

	mreq := &pb_v1.GroupAddRequest{IsSuper:true, Cid:req.Cid, GardenId:req.GardenId, GroupName:"超级管理员", GroupDesc:"超级管理员组"}
	mreply, err := GroupAdd(context.Background(), mreq)
	if err != nil {
		return nil, err
	}
	reply.Id = mreply.Id

	return reply, nil
}

