// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package spackage

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"time"
)

func checkPackageOrderApproveParam(req *pb_v1.PackageOrderApproveRequest) error {
	switch {
	case req.OrderId < 1:
		return status.Error(10003, "id不能为空")

	}
	return nil
}

func UpdateGarden(gardenId int64, period int64, db *gorm.DB) error {
	p := dbmodel.TGardenApproved{}
	where := map[string]interface{}{
		"id": gardenId,
	}
	err := p.Find(db, where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return errors.DataBaseError
	}
	if p.ID == 0 {
		return errors.ErrRecordNotFound
	}

	now := time.Now()
	now = time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location())
	expire := int64(0)
	if p.ExpireAt > now.Unix() {
		expire = time.Unix(p.ExpireAt, 0).AddDate(int(period), 0, 0).Unix()
	} else {
		expire = now.AddDate(int(period), 0, 0).Unix()
	}

	values := map[string]interface{}{
		"expire_at": expire,
	}

	err = p.Update(db, where, values)
	if err != nil {
		return errors.DataBaseError
	}
	return nil
}

func ApproveHandle(req *pb_v1.PackageOrderApproveRequest, db *gorm.DB) error {
	p := dbmodel.TPackageOrder{}
	where := map[string]interface{}{
		"id": req.OrderId,
	}
	err := p.Find(db, where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return errors.DataBaseError
	}
	if p.ID == 0 {
		return errors.ErrRecordNotFound
	}
	if p.Status != 1 {
		return status.Error(10003, "状态错误")
	}
	status := 2
	if !req.Status {
		status = 3
	}
	values := map[string]interface{}{
		"status":   status,
		"feedback": req.Feedback,
	}
	p.ID = 0
	err = p.Update(db, where, values)
	if err != nil {
		return errors.DataBaseError
	}

	if p.OrderType == 2 && p.GardenId > 0 {
		return UpdateGarden(p.GardenId, p.Period, db)
	}
	return nil

}

//
func PackageOrderApprove(ctx context.Context, req *pb_v1.PackageOrderApproveRequest) (reply *pb_v1.PackageOrderApproveReply, err error) {

	reply = &pb_v1.PackageOrderApproveReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkPackageOrderApproveParam(req)
	if err != nil {
		return nil, err
	}

	db := database.DB().Begin()
	if err = ApproveHandle(req, db); err != nil {
		db.Rollback()
		return nil, err
	}
	db.Commit()
	return reply, nil
}
