// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package user

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/util"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-system/consts"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"time"
)

func checkChangePasswordParam(req *pb_v1.ChangePasswordRequest) error {
	switch {
	case req.Uid == 0:
		return status.Error(10003, "账号不能为空")
	case req.OldPassword == "":
		return status.Error(10003, "原密码不能为空")
	case req.NewPassword == "":
		return status.Error(10003, "密码不能为空")
	}
	return nil
}

//
func ChangePassword(ctx context.Context, req *pb_v1.ChangePasswordRequest) (reply *pb_v1.ChangePasswordReply, err error) {

	reply = &pb_v1.ChangePasswordReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkChangePasswordParam(req)
	if err != nil {
		return nil, err
	}
	if req.ByCompany {
		return nil, status.Error(10003, "不能进行该操作")
	}


	user := &dbmodel.TUser{}
	where := map[string]interface{}{
		"id":req.Uid,
	}
	err = user.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound{
		return nil, errors.DataBaseError
	}
	if user.ID == 0 {
		return nil, errors.UserNotExist
	}

	oldpasswd, _ := util.GetMd5Pass(req.OldPassword, consts.CRYPTO_KEY)
	if oldpasswd != user.Password {
		return nil, errors.PasswordError
	}
	passwd, _ := util.GetMd5Pass(req.NewPassword, consts.CRYPTO_KEY)

	now := time.Now()
	values := map[string]interface{}{
		"password":passwd,
		"updated_at":now,
	}
	err = user.Update(database.DB(), where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}


	return reply, nil
}
