// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package user

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-system/errors"
	"property-system/impl/v1/permission"
	pb_v1 "property-system/pb/v1"
)

//
func CompanyEnterGarden(ctx context.Context, req *pb_v1.CompanyEnterGardenRequest) (reply *pb_v1.CompanyEnterGardenReply, err error) {

	reply = &pb_v1.CompanyEnterGardenReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.Cid < 1 || req.GardenId < 1 {
		return reply, errors.ParamsError
	}

	// 获取小区信息
	gardenInfo, err := getGardenInfo(req.GardenId)
	if err != nil {
		return nil, err
	}
	if !gardenInfo.InUse {
		return nil, status.Error(10003, "小区已删除")
	}
	if gardenInfo.Cid != req.Cid {
		return nil, status.Error(10004, "小区与公司不一致")
	}
	// 获取权限信息
	gardenPermissionTree := permission.GardenPermissionTreeFromRedis(gardenInfo.ID)
	if len(gardenPermissionTree) == 0 {
		gardenPermissionTree, err = permission.GardenPermissionTreeFromDb(gardenInfo.ID)
		if err != nil {
			return nil, err
		}
	}
	if err != nil {
		return nil, err
	}
	reply.GardenName = gardenInfo.GardenName
	reply.Permissions = makeSuperGroupPermission(gardenPermissionTree)
	reply.GlobalPermissionTime = permission.GloablePermissionTime(gardenInfo.ID)

	return reply, nil
}
