// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package user

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)


//
func UserDel(ctx context.Context, req *pb_v1.UserDelRequest) (reply *pb_v1.UserDelReply, err error) {

	reply = &pb_v1.UserDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.Id < 1 || req.Cid < 1{
		return nil, errors.ParamsError
	}
	if !req.ByCompany && req.Uid < 1 {
		return nil, errors.ParamsError
	}
	if !req.ByCompany && req.GardenId < 1 {
		return nil, errors.ParamsError
	}

	p := &dbmodel.TUser{}
	where := map[string]interface{}{
		"id":req.Id,
		"cid":req.Cid,
	}
	if req.GardenId > 0 {
		where["garden_id"] = req.GardenId
	}
	src := &dbmodel.TUser{}
	err = src.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if src.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	if req.Uid == src.ID && !req.ByCompany{
		return nil, errors.UserSelfError
	}

	reply.Origin = &pb_v1.UserUpdateRequest{
		Id:req.Id,
		Cid:req.Cid,
		GardenId: src.GardenId,
		GroupId:src.GroupId,
		UserName:src.User,
		Email:src.Email,
		Phone:src.Phone,
	}


	err = p.Delete(database.DB(), where)


	if err != nil {
		return nil, errors.DataBaseError
	}

	return reply, nil
}

