// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package user

import (
	"context"
	"encoding/json"
	"fmt"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"time"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)


// 废弃
func UserGardenChange(ctx context.Context, req *pb_v1.UserGardenChangeRequest) (reply *pb_v1.UserGardenChangeReply, err error) {

	reply = &pb_v1.UserGardenChangeReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.GardenId < 1 || req.GardenName == "" {
		return nil, errors.ParamsError
	}


	now := time.Now()

	where := map[string]interface{}{
		"garden_id":req.GardenId,
	}
	values := map[string]interface{}{
		"updated_at":now,
		"garden_name":req.GardenName,
	}

	p := dbmodel.TUser{}
	err = p.Update(database.DB(), where, values)

	if err != nil {

		return nil, errors.DataBaseError
	}



	return reply, nil
}

